within Modelica_DeviceDrivers.Blocks;
package OperatingSystem

  block SynchronizeRealtime "A pseudo realtime synchronization"
    extends Modelica_DeviceDrivers.Utilities.Icons.BaseIcon;
    parameter Integer resolution(min = 1) = 1 "resolution of the timer";
    parameter
      Modelica_DeviceDrivers.Blocks.OperatingSystem.Types.ProcessPriority
      priority = "Normal" "Priority of the simulation process";
    output Real calculationTime "Time needed for calculation";
    output Real availableTime
      "Time available for calculation (integrator step size)";
  equation
    when (initial()) then
      Modelica_DeviceDrivers.OperatingSystem.setProcessPriority(
        if
          (priority == "Idle") then -2 else
        if
          (priority == "Below normal") then -1 else
        if
          (priority == "Normal") then 0 else
        if
          (priority == "High priority") then 1 else
        if
          (priority == "Realtime") then 2 else
        0);
    end when;

     (calculationTime,availableTime) = Modelica_DeviceDrivers.OperatingSystem.realtimeSynchronize(time,resolution);
    annotation (Icon(coordinateSystem(preserveAspectRatio=true,  extent={{-100,
              -100},{100,100}}), graphics={
          Bitmap(extent={{-60,60},{60,-60}}, fileName=
                "../Resources/Images/Icons/clock.png"),
          Text(
            extent={{-100,140},{100,100}},
            lineColor={0,0,255},
            textString="%name"),
          Text(
            extent={{-100,-100},{100,-140}},
            lineColor={0,0,0},
            textString="%priority")}));
  end SynchronizeRealtime;

  package Types

  type ProcessPriority = Modelica.Icons.TypeString
  annotation (
    preferedView="text",
    Evaluate=true,
    choices(
      choice="Idle" "Idle",
      choice="Below normal" "Below normal",
      choice="Normal" "Normal",
      choice="High priority" "High priority",
      choice="Realtime" "Realtime"));
  end Types;
end OperatingSystem;
