within Modelica_DeviceDrivers.OperatingSystem;
function realtimeSynchronize
  "pauses the simulation until synchronisation with realtime is achieved"

input Real simTime;
input Integer resolution = 1;
output Real calculationTime;
output Real availableTime;
external "C" calculationTime = OS_realtimeSynchronize(simTime,resolution,availableTime);
annotation(Include = "
 
#ifndef MDDSYNC
#define MDDSYNC
#define VOID void
typedef char CHAR;
typedef short SHORT;
typedef long LONG;
typedef unsigned char   u_char;
typedef unsigned short  u_short;
typedef unsigned int    u_int;
typedef unsigned long   u_long;
typedef unsigned __int64 u_int64;
#include <winsock2.h>//for compatibility reasons 
#include <windows.h> 
#include \"ModelicaUtilities.h\"
#pragma comment( lib, \"Winmm.lib\" ) 
double MDD_lastTime = 0;
double MDD_startTime = 0;
double MDD_lastSimTime = 0; 
double MDD_lastAvailableTime = 0; 
 
double OS_getTimeMSsync(int resolution)
{
  DWORD ms;
 
  if(timeBeginPeriod(resolution))
    ModelicaFormatError(\"realtimSynchronize: Resolution not supported by hardware!\");
  ms = timeGetTime();
  timeEndPeriod(resolution);
  return (double)ms;
 
};
 
double OS_realtimeSynchronize(double simTime, int resolution, double * availableTime)
{
  DWORD exitCode= 0;
  
  double calculationTime = 0;
 
  if(MDD_startTime == 0)MDD_startTime = OS_getTimeMSsync(resolution);
  if(MDD_lastTime == 0)MDD_lastTime = OS_getTimeMSsync(resolution);
 
  *availableTime = MDD_lastAvailableTime;
  if(simTime != MDD_lastSimTime)
  {
    
 
    calculationTime = (OS_getTimeMSsync(resolution) - MDD_lastTime)/1000;
 
    *availableTime = simTime - MDD_lastSimTime;
    while((OS_getTimeMSsync(resolution)- MDD_startTime)/1000 <= simTime)
    {
      Sleep(0);
    }
    MDD_lastTime = OS_getTimeMSsync(resolution); 
    MDD_lastSimTime = simTime;
    MDD_lastAvailableTime = *availableTime;
  }
  return calculationTime;
}
 
 
#endif
 
");
end realtimeSynchronize;
