/*                                                         /|
Copyright 2010                                          --/-/--
_______________________________________________________/_/_/__/___
Deutsches Zentrum fr Luft- und Raumfahrt e.V.           |/ DLR
Institut fr Robotik und Mechatronik


MatlabRemoteInterface1.cpp

Autor: Tobias Bellmann
Beschreibung: bla bla bla
Version 0.1
*/
#include "mex.h"
#include "SharedMemoryInterface.h"
#include "SimpleSerialPackager.h"
#include <iostream>

#define BUFFERSIZE 1024 * 1024


/* gateway function */
void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] )
{
	mxArray ** rhs;
	mxArray ** lhs;
    (void) nlhs; (void) plhs;    /* unused parameters */
    (void) nrhs; (void) prhs;

    bool running = true;
	//Access Shared Memory:

	
	
	SimpleSerialPackager ssP(BUFFERSIZE);
	SharedMemoryInterface smI("Global\\MatlabInterface",BUFFERSIZE);
	mexPrintf("Shared Memory opened, waiting for function calls...\n");
	mexEvalString("drawnow;");
	while(1)
	{
		smI.read(ssP.getPackage(),BUFFERSIZE);
		ssP.resetPointer();
		int control = 0;
		// Wait for data to be processed:
		while(control == 0)
		{
			smI.read(ssP.getPackage(),BUFFERSIZE);
			ssP.resetPointer();
			ssP.getInteger(&control,1,1);
			Sleep(0);
		}
		// check if abort condition is set:

		if(control ==99)
			return;
		
		// Start processing the data:
		
		// get number of inputs and outputs:
		int numInputs, numOutputs;
		ssP.getInteger(&numInputs,1,1);
		ssP.getInteger(&numOutputs,1,1);
		rhs = (mxArray**)mxMalloc(sizeof(void*) * numInputs);
		lhs = (mxArray**)mxMalloc(sizeof(void*) * numOutputs);
		
		// get name of Matlab function:
		char * matlabFunctionName[64];
		ssP.getString((char*)matlabFunctionName);
		// allocate and fill matlab matrices:

		for(int i = 0; i < numInputs; i++)
		{
			int m, n,type;
			

			ssP.getDimensionsAndType(&m,&n,&type);

			if(type!= sizeof(double)) 
				return;

			rhs[i] = mxCreateDoubleMatrix(m,n, mxREAL);
			ssP.getDouble(mxGetPr(rhs[i]),m,n);
			
		
		}
		
		// call the Matlab function:
		if(numOutputs == 0)
			mexCallMATLAB(0, NULL, numInputs, rhs, (char*)matlabFunctionName);
		else
			mexCallMATLAB(numOutputs, lhs, numInputs, rhs, (char*)matlabFunctionName);

		// clear the package to refill it with the results
		ssP.clear();
		control = 0;
		//set the cleared control bit:		
		ssP.addInteger(&control,1,1);
		
		// write result to package:
		for(int i = 0; i < numOutputs; i++)
		{
			//mexPrintf("Writing Result to package.\n");
			//mexEvalString("drawnow;");
			int m = mxGetM(lhs[i]);
			int n = mxGetN(lhs[i]);
			ssP.addDouble(mxGetPr(lhs[i]),m,n);

		}
		//mexPrintf("Writing package to shared memory\n");
		//mexEvalString("drawnow;");
		// write package to shared memory:
		smI.write(ssP.getPackage(),BUFFERSIZE);
		

		
	}
	
	return;
}
