#include <iostream>
#include "SimpleSerialPackager.h"
SimpleSerialPackager::SimpleSerialPackager(int bufferSize)
{
	_buffer = new char[bufferSize];
	_bufferSize = bufferSize;
	_pos = 0;
}

void SimpleSerialPackager::addDouble(double *u, int m, int n)
{
	int typeSize = sizeof(double);
	//check buffersize
	if(_pos + 3 * sizeof(int) + typeSize * m * n >= _bufferSize)
	{
		std::cout << "SimpleSerialPackager: Buffer overflow." << std::endl;
		return;
	}

	//copy data type size in buffer
	memcpy(_buffer + _pos, &typeSize, sizeof(int));
	_pos += sizeof(int);

	//copy n dim of data in buffer
	memcpy(_buffer + _pos, &m, sizeof(int));
	_pos += sizeof(int);

	//copy m dim of data in buffer
	memcpy(_buffer + _pos, &n, sizeof(int));
	_pos += sizeof(int);

	//copy data in buffer
	memcpy(_buffer + _pos, u, typeSize * n * m);
	_pos += typeSize * n * m;
}
void SimpleSerialPackager::addInteger(int *u, int m, int n)
{
	int typeSize = sizeof(int);
	//check buffersize
	if(_pos + 3 * sizeof(int) + typeSize * m * n >= _bufferSize)
	{
		std::cout << "SimpleSerialPackager: Buffer overflow." << std::endl;
		return;
	}

	//copy data type size in buffer
	memcpy(_buffer + _pos, &typeSize, sizeof(int));
	_pos += sizeof(int);

	//copy n dim of data in buffer
	memcpy(_buffer + _pos, &m, sizeof(int));
	_pos += sizeof(int);

	//copy m dim of data in buffer
	memcpy(_buffer + _pos, &n, sizeof(int));
	_pos += sizeof(int);

	//copy data in buffer
	memcpy(_buffer + _pos, u, typeSize * n * m);
	_pos += typeSize * n * m;
}
void SimpleSerialPackager::addString(char * u)
{
  int typeSize = sizeof(char);
  int len = 0; 
   
  //copy data type size in buffer
  memcpy(_buffer + _pos, &typeSize, sizeof(int));
  _pos += sizeof(int);
 
  //find zero terminated end of string
  len = strlen(u); 
 
  //copy data in buffer
  memcpy(_buffer + _pos, u, typeSize * len);
  _pos += typeSize * len;
 
}
void SimpleSerialPackager::getDouble(double *y, int m, int n)
{
  int m_buf;
  int n_buf;
  int typeSize_buf;
  int typeSize = sizeof(double); 
  //check type
  memcpy(&typeSize_buf, _buffer+ _pos, sizeof(int));
  _pos+= sizeof(int);
 
 
  if(typeSize_buf != typeSize)
	  std::cout << "SSP_getReal: Typesize error: read Type: "<< typeSize_buf << ",  demanded Type "<< typeSize << std::endl;  
  
  //check dimensions
  memcpy(&m_buf, _buffer+ _pos,sizeof(int));
  _pos += sizeof(int);
  memcpy(&n_buf, _buffer+ _pos,sizeof(int));
  _pos += sizeof(int);
 
  if(m_buf!=m || n_buf != n)
	  std::cout << "SSP_getReal: Dimension error: read Dim: "<<m_buf << " , " <<n_buf<< ";  demanded Dim << " <<m << "," <<n << std::endl;
 
  //get data
  memcpy(y,_buffer+ _pos,typeSize * m * n);
  _pos+=typeSize * m * n;
 
 
}

void SimpleSerialPackager::getInteger(int *y, int m, int n)
{
	int m_buf;
	int n_buf;
	int typeSize_buf;
	int typeSize = sizeof(int); 
	//check type
	memcpy(&typeSize_buf, _buffer+ _pos, sizeof(int));
	_pos+= sizeof(int);


	if(typeSize_buf != typeSize)
	{
		//std::cout << "SSP_getInteger: Typesize error: read Type: "<< typeSize_buf << ",  demanded Type "<< typeSize << std::endl;  
		return;
	}
	//check dimensions
	memcpy(&m_buf, _buffer+ _pos,sizeof(int));
	_pos += sizeof(int);
	memcpy(&n_buf, _buffer+ _pos,sizeof(int));
	_pos += sizeof(int);

	if(m_buf!=m || n_buf != n)
	{
		//std::cout << "SSP_getInteger: Dimension error: read Dim: "<<m_buf << " , " <<n_buf<< ";  demanded Dim << " <<m << "," <<n << std::endl;
		return;
	}
	 //get data
	memcpy(y,_buffer+ _pos,typeSize * m * n);
	_pos+=typeSize * m * n;
 
 
}
void SimpleSerialPackager::getString( char * y)
{
  int len = 0;
  int i;
  int typeSize_buf;
  int typeSize = sizeof(char); 
 
  
 
  //check type
  memcpy(&typeSize_buf, _buffer+ _pos, sizeof(int));
  _pos+= sizeof(int);
  if(typeSize_buf != typeSize)
        std::cout << "SSP_getString: Typesize error: read Type: "<< typeSize_buf << ",  demanded Type "<< typeSize << std::endl;  
 
  
  //find zero terminated end of string
  for( i = _pos; i < _bufferSize; i++)
    if (_buffer[i] == 0) break;    

  //get data
  memcpy(y,_buffer+ _pos,typeSize *(i - _pos + 1));
  _pos+=typeSize *(i - _pos + 1);
}
void SimpleSerialPackager::getDimensionsAndType(int * m, int * n, int * type)
{
	
	memcpy(type,_buffer+ _pos,sizeof(int));
	_pos += sizeof(int);
	memcpy(m,_buffer+ _pos,sizeof(int));
	_pos += sizeof(int);
	memcpy(n,_buffer+ _pos,sizeof(int));
	_pos -= 2 * sizeof(int);
	
}


char * SimpleSerialPackager::getPackage()
{
	return _buffer;
}
void SimpleSerialPackager::setPackage(char *buffer, int bufferSize)
{
	_buffer = buffer;
	_bufferSize = bufferSize;
	_pos = 0;
}
void SimpleSerialPackager::clear()
{
	memset(_buffer,0,_bufferSize);
	_pos = 0;
}